#include "mbed.h"
#include "SSD1331.h"

// Image exported from GIMP as RGB565 "*.c" (no modifications needed)
#include "lev.h"
#include "testimg.h"

Serial uart(USBTX, USBRX);
void read_uart();

//           PA8 PB4  PA11 PB5      PB3
//                         MOSI     SCK
SSD1331 oled(D9, D12, D10, D11, NC, D13);

int main() {    
    uart.attach(&read_uart, Serial::RxIrq);
    
    // Example of showing an image.
    // NOTE: This is slow & inefficient, the coordinates are sent for each pixel separately
    // The display should support auto-increment, but it's organized in columns
    // We could also try to use faster SPI.
    int cnt = 0;
    for (int y = 0; y < lev.height; y++) {
        for (int x = 0; x < lev.width; x++) {
            uint16_t c = lev.pixel_data[cnt] | (lev.pixel_data[cnt+1]<<8);
            oled.drawPixel(x, y, c); 
            cnt += 2; 
        }
    }
    
    // Check the library ("SSD1331.h") for other available functions.
    // There are special commands for fast rectangle / line drawing, for example.
}

// Pressing Q-A W-S E-D increases / decreases contrast of color channels
// Example of performing color calibration.
// The default values are set in the library init() function

volatile static uint8_t ca = 0x91;
volatile static uint8_t cb = 0x50;
volatile static uint8_t cc = 0x72;

void read_uart() {
    char c = uart.getc();
    
    switch(c) {
        case 'a':
            if(ca>0) ca--;
            oled.sendCmd(CMD_SET_CONTRAST_A, ca);
            break;
        case 'q':
            if(ca<255) ca++;
            oled.sendCmd(CMD_SET_CONTRAST_A, ca);
            break;
        case 's':
            if(cb>0) cb--;
            oled.sendCmd(CMD_SET_CONTRAST_B, cb);
            break;
        case 'w':
            if(cb<255) cb++;
            oled.sendCmd(CMD_SET_CONTRAST_B, cb);
            break;
        case 'd':
            if(cc>0) cc--;
            oled.sendCmd(CMD_SET_CONTRAST_C, cc);
            break;
        case 'e':
            if(cc<255) cc++;
            oled.sendCmd(CMD_SET_CONTRAST_C, cc);
            break;        
    } 
    
    uart.printf("\rA %d, B %d, C %d            ", ca, cb, cc);           
}
